unit MabLIDlg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, DB, DBTables, MabObj;

type
   TdlgMabLogin = class(TForm)
      btnCancel: TBitBtn;
      btnOk: TBitBtn;
      srcUser: TDataSource;
    grpMain: TGroupBox;
    lblUser: TLabel;
    edtUser: TEdit;
    lblPassword: TLabel;
    edtPassword: TEdit;
      procedure btnOkClick(Sender: TObject);
   private
      { Private declarations }
      UserTable: TTable;
      PasswordField: string;

      procedure SetButtonStyle(AButtonStyle: TButtonStyle);
      procedure SetUserEdit(AUserEdit: TMabEdit);
      procedure SetPasswordEdit(APasswordEdit: TMabPasswordEdit);
   public
      { Public declarations }
      constructor Create(AOwner: TComponent; AUserTable: TTable;
         APasswordField: string);
      function Execute: Boolean;

      property ButtonStyle: TButtonStyle write SetButtonStyle;
      property UserEdit: TMabEdit write SetUserEdit;
      property PasswordEdit: TMabPasswordEdit write SetPasswordEdit;
   end;

   TLoginControl = (lcUser, lcPassword, lcOk, lcCancel);
   TMabDBLoginDlg = class(TComponent)
   private
      { Private declarations }
      FActiveControl: TLoginControl;
      FButtonStyle: TButtonStyle;
      FCaption: string;
      FFont: TFont;
      FParent: TForm;
      FParentFont: Boolean;
      FPasswordEdit: TMabPasswordEdit;
      FPasswordField: string;
      FUserEdit: TMabEdit;
      FUserTable: TTable;

      procedure SetFont(AFont: TFont);
      procedure FontChange(Sender: TObject);
      procedure SetParentFont(AParentFont: Boolean);
      procedure SetPasswordField(APasswordField: string);
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;

      function Execute: Boolean;

      property Parent: TForm read FParent;
   published
      { Published declarations }
      property ActiveControl: TLoginControl read FActiveControl
         write FActiveControl default lcUser;
      property ButtonStyle: TButtonStyle read FButtonStyle write FButtonStyle
         default bsAutoDetect;
      property Caption: string read FCaption write FCaption;
      property Font: TFont read FFont write SetFont;
      property ParentFont: Boolean read FParentFont write SetParentFont
         default True;
      property PasswordEdit: TMabPasswordEdit read FPasswordEdit
         write FPasswordEdit;
      property PasswordField: string read FPasswordField write SetPasswordField;
      property UserEdit: TMabEdit read FUserEdit write FUserEdit;
      property UserTable: TTable read FUserTable write FUserTable;
   end;

implementation

{$R *.DFM}

constructor TdlgMabLogin.Create(AOwner: TComponent;
      AUserTable: TTable; APasswordField: string);
begin
   inherited Create(AOwner);

   srcUser.DataSet := AUserTable;
   PasswordField := APasswordField;

   edtUser.Height := 0;
   edtPassword.Height := 0;
end;

procedure TdlgMabLogin.SetButtonStyle(AButtonStyle: TButtonStyle);
begin
   btnOk.Style := AButtonStyle;
   btnCancel.Style := AButtonStyle;
end;

procedure TdlgMabLogin.SetUserEdit(AUserEdit: TMabEdit);
begin
   with AUserEdit do
   begin
      lblUser.Caption := Prompt;
      edtUser.CharCase := CharCase;
      edtUser.MaxLength := MaxLength;
      edtUser.Text := Text;
   end;
end;

procedure TdlgMabLogin.SetPasswordEdit(APasswordEdit: TMabPasswordEdit);
begin
   with APasswordEdit do
   begin
      lblPassword.Caption := Prompt;
      edtPassword.CharCase := CharCase;
      edtPassword.MaxLength := MaxLength;
      edtPassword.PasswordChar := PasswordChar;
      edtPassword.Text := Text;
   end;
end;

function TdlgMabLogin.Execute: Boolean;
begin
   Result := (ShowModal = mrOk);
end;

procedure TdlgMabLogin.btnOkClick(Sender: TObject);
begin
   if not (srcUser.DataSet as TTable).FindKey([edtUser.Text]) then
   begin
      Application.MessageBox('User not found!', 'Login', MB_ICONEXCLAMATION);
      ActiveControl := edtUser;
      edtUser.SelectAll;
      ModalResult := mrNone;
   end
   else
      if srcUser.DataSet.FieldByName(PasswordField).AsString <> edtPassword.Text then
      begin
      Application.MessageBox('Incorrect password!', 'Login', MB_ICONEXCLAMATION);
         ActiveControl := edtPassword;
         edtPassword.SelectAll;
         ModalResult := mrNone;
      end
      else
         ModalResult := btnOk.ModalResult;
end;

constructor TMabDBLoginDlg.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   if Owner is TForm then
      FParent := TForm(Owner)
   else
      raise Exception.Create('Owner must be of type TForm');

   FActiveControl := lcUser;
   FButtonStyle := bsAutoDetect;
   FCaption := 'Login';

   FFont := TFont.Create;
   FFont.Assign(Parent.Font);
   FFont.OnChange := FontChange;
   FParentFont := True;

   FPasswordEdit := TMabPasswordEdit.Create;
   FPasswordEdit.Prompt := '&Password:';

   FPasswordField := 'Password';

   FUserEdit := TMabEdit.Create;
   FUserEdit.Prompt := '&User:';
end;

destructor TMabDBLoginDlg.Destroy;
begin
   FFont.Free;
   FPasswordEdit.Free;
   FUserEdit.Free;

   inherited Destroy;
end;

procedure TMabDBLoginDlg.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = UserTable) then
      UserTable := nil;
end;

procedure TMabDBLoginDlg.SetFont(AFont: TFont);
begin
   if AFont <> FFont then
      FFont.Assign(AFont);
end;

procedure TMabDBLoginDlg.FontChange(Sender: TObject);
begin
   ParentFont := False;
end;

procedure TMabDBLoginDlg.SetParentFont(AParentFont: Boolean);
begin
   if AParentFont <> ParentFont then
   begin
      if AParentFont then
         Font := Parent.Font;
      FParentFont := AParentFont;
   end;
end;

procedure TMabDBLoginDlg.SetPasswordField(APasswordField: string);
begin
   if (UserTable <> nil) and UserTable.Active
   and (UserTable.FindField(APasswordField) = nil) then
      raise EDatabaseError.Create('Field does not exist in UserTable')
   else
      FPasswordField := APasswordField
end;

function TMabDBLoginDlg.Execute: Boolean;
var
   Bookmark: TBookmark;
   LoginDlg: TdlgMabLogin;
begin
   if UserTable = nil then
      raise EDatabaseError.Create('UserTable must be set')
   else
   begin
      Result := False;
      Bookmark := UserTable.GetBookmark;

      UserTable.FindKey([UserEdit.Text]);
      LoginDlg := TdlgMabLogin.Create(Self, UserTable, PasswordField);
      with LoginDlg do
         try
            case Self.ActiveControl of
               lcUser:     ActiveControl := edtUser;
               lcPassword: ActiveControl := edtPassword;
               lcOk:       ActiveControl := btnOk;
               lcCancel:   ActiveControl := btnCancel;
            end;
            ButtonStyle  := Self.ButtonStyle;
            Caption      := Self.Caption;
            PasswordEdit := Self.PasswordEdit;
            UserEdit     := Self.UserEdit;

            Font.Name := ' ';
            Font.Assign(Self.Font);

            Result       := Execute;
         finally
            Free;
         end;

      if not Result then
         UserTable.GotoBookmark(Bookmark);
      UserTable.FreeBookmark(Bookmark);
   end;
end;

end.
